
; ========================================================
;			
;    -- Jumclass.asm --
;			Jay Sullivan
; ========================================================
  

.386
.MODEL FLAT,STDCALL
OPTION CASEMAP:NONE

		
;#################################################
;#                INCLUDE
;#################################################
		
		include Jumber2.inc
		include Jumclass.inc
		
;#################################################
;#                .CODE
;#################################################
.CODE




;---------------------------------------------
; Function: Jumber_NewRadix
; This function starts a new radix
;---------------------------------------------
Jumber_NewRadix PROC SizeBytes:DWORD

	;---------------------------------------------
	; Free if already exists
	;---------------------------------------------
	.if pGlobalRadix != 0
		invoke HeapFree, hHeap, 0, pGlobalRadix
		mov pGlobalRadix, 0
	.endif

	;---------------------------------------------
	; SizeGlobalRadix = SizeBytes
	;---------------------------------------------
	mov eax, SizeBytes
 	mov SizeGlobalRadix, eax
 	
	;---------------------------------------------
	; Alloc new Radix
	;---------------------------------------------
	invoke HeapAlloc, hHeap,HEAP_ZERO_MEMORY, eax
 	mov pGlobalRadix, eax
 	
 	
	ret
Jumber_NewRadix ENDP

;---------------------------------------------
; Function: Jumber_NewText
; This function starts a new text
;---------------------------------------------
Jumber_NewText PROC SizeBytes:DWORD

	;---------------------------------------------
	; Free if already exists
	;---------------------------------------------
	.if pGlobalText != 0
		invoke HeapFree, hHeap, 0, pGlobalText
		mov pGlobalText, 0
	.endif

	;---------------------------------------------
	; SizeGlobalText = SizeBytes
	;---------------------------------------------
	mov eax, SizeBytes
 	mov SizeGlobalText, eax
 	
	;---------------------------------------------
	; Alloc new Text
	;---------------------------------------------
	invoke HeapAlloc, hHeap,HEAP_ZERO_MEMORY, eax
 	mov pGlobalText, eax
 	
 	
	ret
Jumber_NewText EndP


;---------------------------------------------
; Function: Jumber_NewTally
; This function starts a new Tally
;---------------------------------------------
Jumber_NewTally PROC

	;---------------------------------------------
	; Free if already exists
	;---------------------------------------------
	.if pGlobalTally != 0
		invoke HeapFree, hHeap, 0, pGlobalTally
		mov pGlobalTally, 0
	.endif
 	
	;---------------------------------------------
	; Alloc new Tally
	;---------------------------------------------
	invoke HeapAlloc, hHeap,HEAP_ZERO_MEMORY, 4096*8
 	mov pGlobalTally, eax
 	
 	
	ret
Jumber_NewTally EndP



;---------------------------------------------
; Function: Jumber_FreeRadix
; This function frees the old Radix
;---------------------------------------------
Jumber_FreeRadix PROC
	
	;---------------------------------------------
	; Make sure it already exists
	;---------------------------------------------
	.if pGlobalRadix != 0
		invoke HeapFree, hHeap, 0, pGlobalRadix
		mov pGlobalRadix, 0
	.endif
 	mov SizeGlobalRadix, 0
	
	ret
Jumber_FreeRadix ENDP

;---------------------------------------------
; Function: Jumber_FreeText
; This function frees the old Text
;---------------------------------------------
Jumber_FreeText PROC
	
	;---------------------------------------------
	; Make sure it already exists
	;---------------------------------------------
	.if pGlobalText != 0
		invoke HeapFree, hHeap, 0, pGlobalText
		mov pGlobalText, 0
	.endif
 	mov SizeGlobalText, 0
	
	ret
Jumber_FreeText ENDP

;---------------------------------------------
; Function: Jumber_FreeTally
; This function frees the old Tally
;---------------------------------------------
Jumber_FreeTally PROC
	
	;---------------------------------------------
	; Make sure it already exists
	;---------------------------------------------
	.if pGlobalTally != 0
		invoke HeapFree, hHeap, 0, pGlobalTally
		mov pGlobalTally, 0
	.endif
 	mov SizeGlobalTally, 0
	
	ret
Jumber_FreeTally ENDP




;---------------------------------------------
; Function: Jumber_IsZero
; Is the tally equal to zero?
;---------------------------------------------
Jumber_IsZero PROC
 mov ecx, SizeGlobalTally
 mov eax, pGlobalTally
 mov eax, dword ptr [eax]
.IF eax==0 && ecx==0
	mov eax, TRUE
.ELSE
	mov eax, FALSE
.ENDIF
 ret
Jumber_IsZero EndP


;---------------------------------------------
; Function: Jumber_IsOne
; Is the tally <= 1?
;---------------------------------------------
Jumber_IsOne PROC
 mov ecx, SizeGlobalTally
 mov eax, pGlobalTally
 mov eax, dword ptr [eax]
.IF eax<=1 && ecx==0
	mov eax, TRUE
.ELSE
	mov eax, FALSE
.ENDIF
 ret
Jumber_IsOne EndP

;---------------------------------------------
; Function: Jumber_GetSize
; Get size of tally
;---------------------------------------------
Jumber_GetSize PROC
 mov eax, SizeGlobalTally
 ret
Jumber_GetSize EndP

;---------------------------------------------
; Function: Jumber_Get
; Get tally[n]
;---------------------------------------------
Jumber_Get PROC INDEX:DWORD
 mov ecx, INDEX
 mov eax, pGlobalTally
 mov eax, dword ptr [eax+4*ecx] 
 ret
Jumber_Get Endp

;---------------------------------------------
; Function: Jumber_Set
; Set a given index of the tally to any number
;---------------------------------------------
Jumber_Set PROC INDEX:DWORD, setend:DWORD
 mov ecx, INDEX
 mov eax, pGlobalTally
 mov edx, setend
 mov dword ptr [eax+ecx*4],  edx  
 ret
Jumber_Set Endp




;---------------------------------------------
; Function: Jumber_Add
; Add number to tally
;---------------------------------------------
Jumber_Add PROC addend:DWORD
   
	xor ecx, ecx
	mov ebx, pGlobalTally
	mov edx, SizeGlobalTally
	mov eax, addend
	
	 add dword ptr [ebx+ecx*4], eax
	 jnc done
	 add ecx, 1
	 sub edx, 1
	 jc newdigit
	
	dorepeat:
	
	 add dword ptr [ebx+ecx*4], 1
	 jnc done
	 add ecx, 1
	 sub edx, 1
	 jnc dorepeat
	
	newdigit:
	 mov dword ptr [ebx+ecx*4], 1
	
	done:
	 mov SizeGlobalTally, edx  ;// EDIT: ecx - > edx
     ret
Jumber_Add EndP

;---------------------------------------------
; Function: Jumber_Multiply
; Multiply tally with number
;---------------------------------------------
Jumber_Multiply PROC  multiplicand:DWORD

		xor ecx,ecx
		xor edx,edx
		mov esi, pGlobalTally
		mov edi, SizeGlobalTally

		
		;//start
		mov eax, [esi+ecx*4]
		mul multiplicand
		mov [esi+ecx*4], eax
			add ecx, 1
			sub edi, 1
			jc done

		;//loop
	dorepeat:
		mov eax, [esi+ecx*4]
		mov [esi+ecx*4], edx
		mul multiplicand
		add [esi+ecx*4], eax
			add ecx, 1
			sub edi, 1
			jnc dorepeat

		;//done
	done:
		test edx, edx
		jz ended
		mov [esi+ecx*4], edx
		mov SizeGlobalTally, ecx
		;//really done
	ended:
	Ret
Jumber_Multiply EndP

;---------------------------------------------
; Function: Jumber_Divide
; Divide tally with number
;---------------------------------------------
Jumber_Divide PROC  divisor:DWORD
		xor edx,edx
		mov esi, pGlobalTally
		mov ecx, SizeGlobalTally
		;//start
			mov  eax, [esi+ecx*4]

			div  divisor
			mov  [esi+ecx*4], eax
			sub ecx,1
			jc done

		;//check
			cmp dword ptr [esi + ecx*4 + 4], 0  ;// EDIT: +4
			jne nextdigit
			mov SizeGlobalTally, ecx

		;//loop
		nextdigit:
			mov  eax, [esi+ecx*4]
			div  divisor
			mov  [esi+ecx*4], eax
			sub ecx,1
			jnc nextdigit

		;//done
		done:
			mov eax, edx
	Ret
Jumber_Divide EndP

;---------------------------------------------
; Function: Jumber_Subtract
; Subtract number from tally
;---------------------------------------------
Jumber_Subtract PROC subtrahend:DWORD
		xor ecx, ecx
		mov ebx, pGlobalTally
		mov edx, SizeGlobalTally
		
		mov eax, subtrahend
		sub dword ptr [ebx+ecx*4], eax
		jnc done
		
		add ecx, 1
		sub edx, 1
		jc newdigit

		dorepeat:

			sub dword ptr [ebx+ecx*4], 1
			jnc done
			add ecx, 1
			sub edx, 1
			jnc dorepeat

		newdigit:
		mov dword ptr [ebx+ecx*4], 1

		done:
		cmp dword ptr [ebx+ecx*4], 0
		jnz keepdigit
		sub ecx, 1
		keepdigit:
		mov SizeGlobalTally, ecx
	Ret
Jumber_Subtract EndP


;---------------------------------------------
; Function: DecipherTally
; Grind tally into string
; Input =
;     pGlobalRadix,  SizeGlobalRadix
;     pGlobalTally,  SizeGlobalTally
; Output = 
;     pGlobalText, SizeGlobalText
;
;---------------------------------------------
DecipherTally PROC
  LOCAL pMyText:DWORD
  LOCAL BaseRadix:DWORD
  		
	;---------------------------------------------
	;  Set pGlobalText
	;---------------------------------------------
		invoke Jumber_NewText, 4096
		
	;----------------------------------------
	;  Adjust radix due to null-terminator
	;----------------------------------------
	mov eax, SizeGlobalRadix
	sub eax, 1
	mov BaseRadix, eax
	
	;---------------------------------------------
	;  Set pMyText
	;---------------------------------------------
	mov ecx, pGlobalText
	mov pMyText, ecx
	
	;---------------------------------------------
	;  Decipher each letter until we hit a 1 (one)
	;---------------------------------------------
	mov eax, 0
	.while eax==0
		;---------------------------------------------
		;  Grind out next letter
		;---------------------------------------------
		invoke Jumber_Divide, BaseRadix
		;---------------------------------------------
		;  Copy pMyRadix[r]
		;---------------------------------------------
		mov ecx, pGlobalRadix
		movzx eax, byte ptr [ecx+eax]  ; get pRadix[r]
		;---------------------------------------------
		;  Paste pMyText[t]
		;---------------------------------------------
		mov ecx, pMyText
		mov byte ptr [ecx], al ; set pText[t]
		;---------------------------------------------
		;  Prepare for next letter
		;---------------------------------------------
		add pMyText,1
		;---------------------------------------------
		;  Stop if total tally is 1
		;---------------------------------------------
		invoke Jumber_IsOne
	.endw

	;---------------------------------------------
	;  Null terminator
	;---------------------------------------------
	mov eax, pMyText
	mov byte ptr [eax], 0
	sub eax, pGlobalText
	mov SizeGlobalText, eax
	
  ret
DecipherTally	EndP



;---------------------------------------------
; Function: GenerateTally
; Uses a specified radix and text to initialize a tally 
; Input =
;     pGlobalRadix,  SizeGlobalRadix
;     pGlobalText,   SizeGlobalText
; Output = 
;     pGlobalTally,  SizeGlobalTally
;---------------------------------------------
GenerateTally PROC uses edi esi ebx
LOCAL BaseRadix:DWORD
  
	;------------------------------
	;  Put leading 1
	;------------------------------
	invoke Jumber_Add, 1

	
	;-----------------------------------
	;  SizeGlobalText indicates the
	;  length of pGlobalText, including
	;  the null terminator.  Thus,
	;  the index of the last non-null
	;  character is (SizeGlobalText-2)
	;-----------------------------------
	mov ebx, SizeGlobalText
	sub ebx, 2
	
	;----------------------------------------
	;  Adjust radix due to null-terminator
	;----------------------------------------
	mov eax, SizeGlobalRadix
	sub eax, 1
	mov BaseRadix, eax
	
	;------------------------------
	;  Set read/write locations
	;------------------------------
	xor eax, eax
	mov edi, pGlobalText
	mov esi, pGlobalRadix
	
	
	;---------------------------------------
	;  Iterate pGlobalText backwards
	;  Find corresponding pGlobalRadix index
	;---------------------------------------
	TestNextTextLetter:
		xor ecx, ecx 				; radix=0
		mov al, byte ptr [edi+ebx]  ; al=text[d]
		
		TryNextRadixDigit:
		
			;---------------------------------------
			;  Did we find the radix digit?
			;---------------------------------------
			cmp al, byte ptr [esi+ecx] ; rad[c] 
			je FoundRadixDigit
			
			;---------------------------------------
			;  Are we at the end of the radix?
			;---------------------------------------
			add ecx, 1 ; c++
			cmp byte ptr [esi+ecx], 0
			je ErrorNonRadixLetter
			
		jmp TryNextRadixDigit
	
		
		;---------------------------------------
		;  Radix digit matches, so
		;  Add it to the tally.
		;---------------------------------------
		FoundRadixDigit:
	  
			push esi
			push edi
			push ebx
			push ecx
			invoke Jumber_Multiply, BaseRadix
			pop ecx
			invoke Jumber_Add, ecx
			pop ebx
			pop edi
			pop esi
			  
		;---------------------------------------
		;  Prepare for next letter.
		;  If we carry, we're done.
		;---------------------------------------
		
			sub ebx, 1
			jc DoneGenerating
		
	jmp TestNextTextLetter
   
   
   
	;---------------------------------------
	; Function failed, return 0
	;---------------------------------------
  	ErrorNonRadixLetter:
		mov eax, 0
		ret
		
	;---------------------------------------
	; Function worked, return 1
	;---------------------------------------
  	DoneGenerating:
		mov eax, 1
		ret
	
GenerateTally EndP






end